/*
 * Created on Jun 29, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

import junit.framework.TestCase;
import java.util.regex.*;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * JUnit test suite for the Grep class.
 * 
 * Exercise the various aspects of the Grep class functionality.
 * 
 * @author rjray
 * @version $Id: GrepTest.java,v 1.3 2004/07/20 08:31:17 rjray Exp $
 */
public class GrepTest extends TestCase
{

    /**
     * Constructor for GrepTest.
     * @param name
     */
    public GrepTest(String name)
    {
        super(name);
    }

    final public void testInstance()
    {
        Grep g1 = Grep.instance();
        Grep g2 = Grep.instance();
        
        assertEquals("Test two calls to instance() for equality", g1, g2);
    }

    /*
     * Test for List grep(GrepBlock, List)
     */
    final public void testGrepGrepBlockList()
    {
        Vector v = new Vector();
        for (int i = 0; i < 100; i++)
        {
            v.add(new Integer(i));
        } 
        List even_numbers = Grep.grep(new GrepBlock() {
            public Object eval(Object I) {
                return Boolean.valueOf((((Integer)I).intValue() & 1) == 0);
            } }, v);
        assertTrue("GrepBlock/List: correct number of elements",
            even_numbers.size() == 50);
        int one_bits = 0;
        for (Iterator iter = even_numbers.iterator(); iter.hasNext(); )
        {
            one_bits += ((Integer)iter.next()).intValue() & 1;
        }
        assertTrue("GrepBlock/List: no odd numbers", one_bits == 0);
    }

    /*
     * Test for List grep(Pattern, List)
     */
    final public void testGrepPatternList()
    {
        String[] lang = { "perl", "python", "ruby", "java", "php", "c" };
        Vector langs = new Vector(Arrays.asList(lang));
        List p_lang = Grep.grep(Pattern.compile("^p"), langs);
        
        assertTrue("Pattern/List: correct number of elements",
            p_lang.size() == 3);
        assertEquals("Pattern/List: element correctness",
            (String)p_lang.get(0), "perl");
    }

    /*
     * Test for List grep(String, List)
     */
    final public void testGrepStringList()
    {
        String[] bWord = { "boo", "bob", "bowl", "brother", "bother" };
        Vector bWords = new Vector(Arrays.asList(bWord));
        List boWords = Grep.grep("bo", bWords);
        
        assertTrue("String/List: correct number of elements",
            boWords.size() == 4);
        assertEquals("String/List: element correctness",
            (String)boWords.get(3), "bother");
    }

    /*
     * Test for Object[] grep(GrepBlock, Object[])
     */
    final public void testGrepGrepBlockObjectArray()
    {
        Vector v = new Vector();
        for (int i = 0; i < 100; i++)
        {
            v.add(new Integer(i));
        } 
        Object[] even_numbers = Grep.grep(new GrepBlock() {
            public Object eval(Object I) {
                return Boolean.valueOf((((Integer)I).intValue() & 1) == 0);
            } }, v.toArray());
        assertTrue("GrepBlock/Array: correct number of elements",
            even_numbers.length == 50);
        int one_bits = 0;
        for (int i = 0; i < even_numbers.length; i++)
        {
            one_bits += ((Integer)even_numbers[i]).intValue() & 1;
        }
        assertTrue("GrepBlock/Array: no odd numbers", one_bits == 0);
    }

    /*
     * Test for Object[] grep(Pattern, Object[])
     */
    final public void testGrepPatternObjectArray()
    {
        String[] lang = { "french", "german", "english", "spanish", "flemish" };
        Object[] p_lang = Grep.grep(Pattern.compile("sh$"), lang);
        
        assertTrue("Pattern/Array: correct number of elements",
            p_lang.length == 3);
        assertEquals("Pattern/Array: element correctness",
            (String)p_lang[0], "english");
    }

    /*
     * Test for Object[] grep(String, Object[])
     */
    final public void testGrepStringObjectArray()
    {
        String[] bWord = { "bath", "bathe", "bar", "bowl", "bat" };
        Object[] baWords = Grep.grep("ba", bWord);
        
        assertTrue("String/List: correct number of elements",
            baWords.length == 4);
        assertEquals("String/List: element correctness",
            (String)baWords[3], "bat");
    }
}
