/*
 * Created on Jun 23, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

import junit.framework.TestCase;

/**
 * JUnit testing-class for the Join class.
 * 
 * This tests the functionality of the <code>Join</code> class, which involves
 * only two methods.
 * 
 * @author rjray
 * @version $Id: JoinTest.java,v 1.6 2004/07/31 12:27:07 rjray Exp $
 */
public class JoinTest extends TestCase
{
    /**
     * Constructor for JoinTest.
     * @param arg0  Identifying string for the test
     */
    public JoinTest(String arg0)
    {
        super(arg0);
    }

    /*
     * Test the instance() method to ensure that it always returns the same
     * object.
     */
    final public void testInstance()
    {
        // Call instance() twice. See if they match.
        Join c1 = Join.instance();
        Join c2 = Join.instance();
        
        assertEquals("Test two calls to instance() for equality", c1, c2);
    }

    /*
     * Test for String join(char, String[]), normal data
     */
    final public void testJoincharStringArray1()
    {
        String list[] = { "a", "b", "c" };
        
        assertEquals("Test join(char, String[]) #1",
            Join.join(',', list), "a,b,c");
    }

    /*
     * Test for String join(char, String[]), empty list
     */
    final public void testJoincharStringArray2()
    {
        String list[] = {};
        
        assertEquals("Test join(char, String[]) #2",
            Join.join(',', list), "");
    }

    /*
     * Test for String join(char, String[]), null or uninitialized list
     */
    final public void testJoincharStringArray3()
    {
        String list[] = null;
        
        assertEquals("Test join(char, String[]) #3",
            Join.join(',', list), "");
    }
    
    // At this point, I'm content with the (char, String[]) signature.

    /*
     * Test for String join(String, String[]), normal data
     */
    final public void testJoinStringStringArray1()
    {
        String list[] = { "a", "b", "c" };
        
        assertEquals("Test join(String, String[]) #1",
            Join.join(",,", list), "a,,b,,c");
    }

    /*
     * Test for String join(String, String[]), empty list
     */
    final public void testJoinStringStringArray2()
    {
        String list[] = {};
        
        assertEquals("Test join(String, String[]) #2",
            Join.join(",,", list), "");
    }

    /*
     * Test for String join(String, String[]), null or uninitialized list
     */
    final public void testJoinStringStringArray3()
    {
        String list[] = null;
        
        assertEquals("Test join(String, String[]) #3",
            Join.join(",,", list), "");
    }
    
    /*
     * Test for String join(String, String[]), data to exceed 1024 characters
     * (the initial allocation for the internal representation).
     */
    final public void testJoinStringStringArray4()
    {
        String replicate = "0123456789abcdef";
        // Use that to make a 128-character string
        String str128 = "";
        // And that will be used to put 8 copies in here
        String[] strArr8 = new String[8];
        // Along the way we will make the whole thing here, as well.
        String whole = "";
        
        int i;
        
        for (i = 0; i < 8; i++)
        {
            str128 += replicate;
        }
        for (i = 0; i < 8; i++)
        {
            strArr8[i] = str128;
            whole += str128 + ",";
        }
        // Drop the trailing comma (no relying on "chop" here)
        whole = whole.substring(0, whole.length() - 1);
        
        assertEquals("Test join(String, String[]) #4",
            Join.join(",", strArr8), whole);
    }
    
    /*
     * Test for a fairly complex join, but with token == null
     */
    final public void testJoinStringStringArray5()
    {
        String replicate = "0123456789abcdef";
        // Use that to make a 128-character string
        String str128 = "";
        // And that will be used to put 8 copies in here
        String[] strArr8 = new String[8];
        // Along the way we will make the whole thing here, as well.
        String whole = "";
        
        int i;
        
        for (i = 0; i < 8; i++)
        {
            str128 += replicate;
        }
        for (i = 0; i < 8; i++)
        {
            strArr8[i] = str128;
            whole += str128;
        }
        
        assertEquals("Test join(String, String[]) #5 (null String)",
            Join.join(null, strArr8), whole);
    }

    /*
     * Test for String join(String, String[]), String is empty (but non-null)
     */
    final public void testJoinStringStringArray6()
    {
        String list[] = { "a", "b", "c" };
        
        assertEquals("Test join(String, String[]) #6 (empty String)",
            Join.join("", list), "abc");
    }    
}
