/*
 * Created on Jun 24, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

import junit.framework.TestCase;

import java.util.Locale;

/**
 * JUnit test class for <code>Lcfirst</code>.
 * 
 * Exercises the various public methods of the <code>Lcfirst</code> class.
 * 
 * @author rjray
 * @version $Id: LcfirstTest.java,v 1.3 2004/07/20 08:31:17 rjray Exp $
 */
public class LcfirstTest extends TestCase
{
    private static final String in  = "FRED";
    private static final String out = "fRED";
    
    /**
     * Constructor for LcfirstTest.
     * @param name
     */
    public LcfirstTest(String name)
    {
        super(name);
    }

    /*
     * Test the instance() method to ensure that it always returns the same
     * object.
     */
    final public void testInstance()
    {
        // Call instance() twice. See if they match.
        Lcfirst c1 = Lcfirst.instance();
        Lcfirst c2 = Lcfirst.instance();
        
        assertEquals("Test two calls to instance() for equality", c1, c2);
    }

    /*
     * Test for String lcfirst(String)
     */
    final public void testLcfirstString()
    {
        assertEquals("Test basic String signature",
            Lcfirst.lcfirst(in), out);
    }

    /*
     * Test for String lcfirst(String, Locale)
     */
    final public void testLcfirstStringLocale()
    {
        // This is kind of a cheat, using the "US" locale...
        assertEquals("Test basic String signature",
            Lcfirst.lcfirst(in, Locale.US), out);
    }

    /*
     * Test for String lcfirst(StringBuffer)
     */
    final public void testLcfirstStringBuffer()
    {
        StringBuffer buffer = new StringBuffer(10);
        buffer.append(in);
        
        String res = Lcfirst.lcfirst(buffer);
        
        assertEquals("Test basic StringBuffer signature (buffer compare)",
            buffer.toString(), out);
        assertEquals("Test basic StringBuffer signature (result compare)",
            res, out);
    }

    /*
     * Test for String lcfirst(StringBuffer, Locale)
     */
    final public void testLcfirstStringBufferLocale()
    {
        // This is kind of a cheat, using the "US" locale...
        StringBuffer buffer = new StringBuffer(10);
        buffer.append(in);
        
        String res = Lcfirst.lcfirst(buffer, Locale.US);
        
        assertEquals("Test basic StringBuffer signature (buffer compare)",
            buffer.toString(), out);
        assertEquals("Test basic StringBuffer signature (result compare)",
            res, out);
    }
}
