/*
 * Created on Jul 8, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

import junit.framework.TestCase;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * JUnit test suite for the Map class.
 * 
 * A set of simple tests using the JUnit framework.
 * 
 * @author rjray
 * @version $Id: MapTest.java,v 1.3 2004/07/20 08:31:17 rjray Exp $
 */
public class MapTest extends TestCase
{

    /**
     * Constructor for MapTest.
     * @param name
     */
    public MapTest(String name)
    {
        super(name);
    }

    final public void testInstance()
    {
        Map m1 = Map.instance();
        Map m2 = Map.instance();
        
        assertEquals("Test two calls to instance() for equality", m1, m2);
    }

    /*
     * Test for List map(MapBlock, List)
     */
    final public void testMapMapBlockList()
    {
        List numbers = new Vector(10);
        List doubled;
        int sum1 = 0, sum2 = 0;
        
        for (int i = 0; i < 10; i++)
        {
            numbers.add(new Integer(i));
            sum1 += i;
        }
        
        doubled = Map.map(new MapBlock() {
            public Object[] eval(final Object o)
            {
                return new Object[] {
                    new Integer(((Integer)o).intValue()),
                    new Integer(((Integer)o).intValue()) };
            } }, numbers);
    
        for (Iterator iter = doubled.iterator(); iter.hasNext(); )
        {
            sum2 += ((Integer)iter.next()).intValue();
        }
        
        assertTrue("MapBlock/List: Test relative size of Lists",
            (numbers.size() * 2) == doubled.size());
        assertTrue("MapBlock/List: Compare sums", (sum1 * 2) == sum2);
    }

    /*
     * Test for Object[] map(MapBlock, Object[])
     */
    final public void testMapMapBlockObjectArray()
    {
        Integer numbers[] = new Integer[10];
        Object doubled[];
        int sum1 = 0, sum2 = 0;
        
        for (int i = 0; i < 10; i++)
        {
            numbers[i] = new Integer(i);
            sum1 += i;
        }
        
        doubled = Map.map(new MapBlock() {
            public Object[] eval(final Object o)
            {
                return new Object[] {
                    new Integer(((Integer)o).intValue()),
                    new Integer(((Integer)o).intValue()) };
            } }, numbers);
    
        for (int i = 0; i < doubled.length; i++)
        {
            sum2 += ((Integer)doubled[i]).intValue();
        }
        
        assertTrue("MapBlock/List: Test relative size of Lists",
            (numbers.length * 2) == doubled.length);
        assertTrue("MapBlock/List: Compare sums", (sum1 * 2) == sum2);
    }

    /*
     * Test for List map(GrepBlock, List)
     */
    final public void testMapGrepBlockList()
    {
        List number = new Vector(100);
        for (int i = 0; i < 100; i++)
        {
            number.add(new Integer(i));
        }
        // Half the elements in number are even, and thus do not have bit0 set
        // Still, count them explicitly in case the above loop changes
        int before_bit0_count = 0;
        for (Iterator iter = number.iterator(); iter.hasNext(); )
        {
            if ((((Integer)iter.next()).intValue() & 1) == 1)
            {
                before_bit0_count++;
            }
        }
        
        // Run it
        List newNumber = Map.map(new GrepBlock() {
            public Object eval(final Object o)
            {
                int newInt = ((Integer)o).intValue() | 1;
                return new Integer(newInt);
            }
        }, number);
        
        int after_bit0_count = 0;
        for (Iterator iter = newNumber.iterator(); iter.hasNext(); )
        {
            if ((((Integer)iter.next()).intValue() & 1) == 1)
            {
                after_bit0_count++;
            }
        }
        
        assertTrue("GrepBlock/List: bit counts differ",
            before_bit0_count != after_bit0_count);
        assertTrue("GrepBlock/List: after-count equals Vector size",
            after_bit0_count == newNumber.size());
    }

    /*
     * Test for Object[] map(GrepBlock, Object[])
     */
    final public void testMapGrepBlockObjectArray()
    {
        String[] words = { "wood", "sheep", "stone", "Settlers of Catan" };
        int sum1 = 0, sum2 = 0;
        Object[] lengths = Map.map(new GrepBlock() {
            public Object eval(final Object o)
            {
                return new Integer(((String)o).length());
            } }, words);
        // Sum the values
        for (int i = 0; i < lengths.length; i++)
        {
            sum1 += words[i].length();
            sum2 += ((Integer)lengths[i]).intValue();
        }
        
        // Tests
        assertEquals("GrepBlock/Array: Correct number of returned items",
            lengths.length, words.length);
        assertEquals("GrepBlock/Array: Test sum of lengths", sum1, sum2);
    }
}
